﻿using Abp.Json;
using Mapster;
using SnapshotWinFormsApp.Application.Dtos;
using SnapshotWinFormsApp.Entities.NewHotline;
using SnapshotWinFormsApp.Entities.OldHotline;
using SnapshotWinFormsApp.Repository;
using SnapshotWinFormsApp.Repository.Interfaces;
using SnapshotWinFormsApp.Tools;
using SqlSugar;
using System.Diagnostics;
using System.Reflection;

namespace SnapshotWinFormsApp.Application.Interfaces;
public class ImportApplicationBase<TSource, TEntity> : ImportApplicationBase<TSource, TEntity, TSource>
    where TSource : OldBaseEntity, new()
    where TEntity : OldIdEntity, new()
{
    public ImportApplicationBase(CreateInstanceInDto inDto) : base(inDto)
    {
    }
}

public class ImportApplicationBase<TSource, TEntity, TMix> : IImportApplication
    where TSource : OldBaseEntity, new()
    where TEntity : OldIdEntity, new()
    where TMix : OldBaseEntity, new()
{
    public readonly ISourceRepository<TSource> _sourceRepo;
    public readonly ITargetRepository<TEntity> _targetRepo;
    public readonly SqlSugarClient _sugarClient;
    public readonly CreateInstanceInDto _instance;
    public readonly ISelectRepository<SystemDicData> _systemDicDataRepo;
    public readonly IList<SystemDicData> JobType;
    public IList<TMix> Sources = new List<TMix>();
    public IList<TEntity> Targets = new List<TEntity>();
    public ImportApplicationBase(CreateInstanceInDto inDto)
    {
        _instance = inDto;
        _sourceRepo = new SourceRepository<TSource>(inDto);
        _targetRepo = new TargetRepository<TEntity>(inDto);
        _systemDicDataRepo = new SelectRepository<SystemDicData>(inDto);
        JobType = _systemDicDataRepo.Queryable().Where(m => m.DicTypeCode == "JobType").ToList();
    }

    public virtual async Task ImportAsync(Action<string> log, CancellationToken token)
    {
        log($"正在查询旧数据...");
        var sql = GetSourceList().ToSqlString();
        Sources = await GetSourceList().ToListAsync(token);

        log($"共查询到{Sources.Count}条数据");
        //var tableName = typeof(TSource).GetCustomAttribute<SugarTable>()?.TableName ?? throw new ArgumentNullException("旧数据表名不能为空, 设置[SugarTable()]");

        for (int i = 0;i < Sources.Count;i++)
        {
            var stopwatch = Stopwatch.StartNew();
            await Task.Run(() => MainForm._pauseEvent.WaitHandle.WaitOne(), token);
            var item = Sources[i];
            item.Id = item.Id.ToString().Trim();

            var has = true;
            try
            {
                has = await HasOldDataAsync(item, token);
            }
            catch (Exception e)
            {

                if (e.Message.Contains("column \"OldId\" does not exist"))
                {
                    var tableName = new TEntity().GetTableName();
                    log($"{tableName}表没有 OldId 字段");
                    log($"ALTER TABLE \"public\".\"{tableName}\" ADD COLUMN \"OldId\" varchar(255);");
                    return;
                }
            }
            if (has)
            {
                var showTimeSkip = stopwatch.ElapsedMilliseconds < 1000 ? stopwatch.ElapsedMilliseconds + "毫" : ((double)stopwatch.ElapsedMilliseconds / 1000).ToString("f2") + "秒";
                log($"{i + 1}/{Sources.Count} {showTimeSkip} 重复: {item.Id}");
                if (i + 1 == Sources.Count)
                {
                    _targetRepo.SubmitBluk();
                    await SubmitBluk();
                }
                continue;
            }
            var target = await GetTargetAsync(item, token);
            if (target == null)
            {
                var showTimeTarget = stopwatch.ElapsedMilliseconds < 1000 ? stopwatch.ElapsedMilliseconds + "毫" : ((double)stopwatch.ElapsedMilliseconds / 1000).ToString("f2") + "秒";
                log($"{i + 1}/{Sources.Count} {showTimeTarget} 获取新对象失败: {item.Id}");
                Logs.Err("获取新对象失败: " + item.ToJson());
                continue;
            }
            target.OldId = item.Id.ToString().Trim();

            var isContinue = await InsertBeforAsync(log, item, target, i + 1 == Sources.Count, token);
            if (isContinue == false)
            {
                var showTimeContinue = stopwatch.ElapsedMilliseconds < 1000 ? stopwatch.ElapsedMilliseconds + "毫" : ((double)stopwatch.ElapsedMilliseconds / 1000).ToString("f2") + "秒";
                log($"{i + 1}/{Sources.Count} {showTimeContinue} 异常: {target.Id} 检查异常日志");
                continue;
            }
            target.Id = _targetRepo.InsertBulk(target, i + 1 == Sources.Count);
            var msg = target.GetObjectValues();
            var showLength = 100;
            if (msg.Length > showLength)
                msg = msg[..showLength];
            await InsertAfterAsync(log, item, target, i + 1 == Sources.Count, token);
            stopwatch.Stop();
            var showTime = stopwatch.ElapsedMilliseconds < 1000 ? stopwatch.ElapsedMilliseconds + "毫" : ((double)stopwatch.ElapsedMilliseconds / 1000).ToString("f2") + "秒";
            log($"{i + 1}/{Sources.Count} {showTime} 新增: {msg}");
        }
        await EndTaskAsync(log, token);
    }

    public virtual async Task SubmitBluk()
    {
        await Task.FromResult(0);
    }

    public virtual async Task<bool> InsertBeforAsync(Action<string> log, TMix item, TEntity target, bool isEnd, CancellationToken token)
    {
        return await Task.FromResult(true);
    }

    public virtual async Task EndTaskAsync(Action<string> log, CancellationToken token)
    {
        await Task.FromResult(0);
    }

    /// <summary>
    /// 插入数据以后执行的动作
    /// </summary>
    /// <param name="item"></param>
    /// <param name="target"></param>
    /// <param name="token"></param>
    /// <returns></returns>
    /// <exception cref="NotImplementedException"></exception>
    public virtual async Task InsertAfterAsync(Action<string> log, TMix item, TEntity target, bool isEnd, CancellationToken token)
    {
        await Task.FromResult(0);
    }

    /// <summary>
    /// 获取原始数据集合
    /// </summary>
    /// <param name="inDto"></param>
    /// <returns></returns>
    public virtual ISugarQueryable<TMix> GetSourceList(string key = "")
    {
        return _sourceRepo.Queryable().Select(m => new TMix(), true);
    }

    /// <summary>
    /// 根据原始数据获取目标数据
    /// </summary>
    /// <param name="source"></param>
    /// <returns></returns>
    public virtual async Task<TEntity> GetTargetAsync(TMix source, CancellationToken token)
    {
        return await Task.FromResult(source.Adapt<TEntity>());
    }

    /// <summary>
    /// 验证是否已存在
    /// </summary>
    /// <param name="tableName"></param>
    /// <param name="item"></param>
    /// <returns></returns>
    public virtual async Task<bool> HasOldDataAsync(TMix item, CancellationToken token)
    {
        return await _targetRepo.Queryable().AnyAsync(m => item.Id.ToString() == m.OldId, token);
    }
}
