﻿using Abp.Extensions;
using Newtonsoft.Json;
using SqlSugar;
using SqlSugar.Extensions;
using System.ComponentModel.DataAnnotations.Schema;
using System.Data;
using System.Reflection;
using System.Security.Cryptography;
using System.Text;
using System.Text.RegularExpressions;

namespace SnapshotWinFormsApp.Tools;

public static class MyExtensions
{
    public static DateTime? ToDate(this string? value)
    {
        if (value == null) return null;
        return value.ObjToDate();
    }

    public static bool IsNullOrTarget(this string? value, string target)
    {
        if (value.IsNullOrEmpty()) return false;
        return value!.Trim() == target;
    }
    public static string MaskPhoneNumber(this string phoneNumber)
    {
        if (String.IsNullOrEmpty(phoneNumber)) return "";

        Regex regex = new Regex(@"(\d{3})(\d{4})(\d{4})");
        Match match = regex.Match(phoneNumber);

        if (!match.Success || match.Groups.Count != 4) return phoneNumber;

        string frontCode = match.Groups[1].Value;
        string areaCode = match.Groups[2].Value;
        string prefixAndSuffix = match.Groups[3].Value;

        string maskedAreaCode = String.Join("", Enumerable.Repeat('*', areaCode.Length));
        string maskedPrefixAndSuffix = String.Join("", Enumerable.Repeat('*', prefixAndSuffix.Length - 4).Concat(prefixAndSuffix.Substring(prefixAndSuffix.Length - 4)));

        return $"{frontCode}{maskedAreaCode}{maskedPrefixAndSuffix}";
    }

    public static string GetObjectValues<T>(this T obj)
    {
        return string.Join(",", obj.GetType()
            .GetProperties(BindingFlags.Public | BindingFlags.Instance)
            .Select(p => p.GetValue(obj)?.ToString())
            .Where(m => m != null));
    }
    public static string ToJson(this object obj)
    {
        return obj == null ? default(string) : JsonConvert.SerializeObject(obj);
    }

    //public static bool IsNullOrEmpty(this string? str)
    //{
    //    return string.IsNullOrEmpty(str);
    //}

    public static bool NotNullOrEmpty(this string? str)
    {
        return !string.IsNullOrEmpty(str);
    }

    /// <summary>
    /// 获取字符串的 md5 
    /// </summary>
    /// <param name="value"> 字符串 </param>
    /// <returns> MD5 后的字符串<see cref="string"/>.  </returns>
    public static string GetMD5(this string value)
    {
        using (var md5 = MD5.Create())
        {
            return BitConverter.ToString(md5.ComputeHash(Encoding.UTF8.GetBytes(value))).Replace("-", string.Empty);
        }
    }

    public static T FromJson<T>(this string json)
    {
        return json.IsNullOrEmpty() ? default(T) : JsonConvert.DeserializeObject<T>(json);
    }

    public static string GetTableName<T>(this T value) where T : class
    {
        var tableAttribute = typeof(T).GetCustomAttribute<SugarTable>();
        return tableAttribute?.TableName ?? string.Empty;
    }

    /// <summary>
    /// 将DataTable转换为指定类型的实体集合
    /// </summary>
    /// <typeparam name="T">实体类型</typeparam>
    /// <param name="dt">DataTable</param>
    /// <returns>实体集合</returns>
    public static List<T> ToList<T>(this DataTable dt) where T : new()
    {
        List<T> list = new List<T>();
        if (dt == null || dt.Rows.Count == 0)
            return list;

        PropertyInfo[] properties = typeof(T).GetProperties(BindingFlags.Public | BindingFlags.Instance | BindingFlags.SetProperty);

        foreach (DataRow row in dt.Rows)
        {
            T item = new T();
            foreach (PropertyInfo property in properties)
            {
                if (dt.Columns.Contains(property.Name) && !row.IsNull(property.Name))
                {
                    object value = Convert.ChangeType(row[property.Name], property.PropertyType);
                    property.SetValue(item, value, null);
                }
            }
            list.Add(item);
        }

        return list;
    }

    public static string ToSqlInsert<T>(this T entity) where T : class
    {
        if (entity == null)
            throw new ArgumentNullException(nameof(entity));

        var properties = typeof(T).GetProperties(BindingFlags.Public | BindingFlags.Instance);

        var tableName = entity.GetTableName();
        var columns = new StringBuilder();
        var values = new StringBuilder();

        foreach (var property in properties)
        {
            if (!property.CanWrite)
                continue;

            var columnName = property.Name;
            var value = property.GetValue(entity);

            if (value == null)
            {
                //values.Append("NULL, ");
                continue;
            }

            if (property.PropertyType == typeof(string) || property.PropertyType == typeof(double))
            {
                value = $"'{value.ToString().Replace("'", "''")}'";
            }
            else if (property.PropertyType == typeof(DateTime) || property.PropertyType == typeof(DateTime?))
            {
                value = $"'{(DateTime)value:yyyy-MM-dd hh:mm:ss.ffffff}'";
            }
            else if (property.PropertyType.BaseType == typeof(Enum))
            {
                value = $"{(int)value}";
            }
            else
            {
                value = value.ToString();
            }

            columns.Append($"\"{columnName}\", ");
            values.Append($"{value}, ");
        }

        if (columns.Length == 0)
            throw new InvalidOperationException("没有可插入的属性。");

        // 移除最后的逗号和空格
        columns.Length -= 2;
        values.Length -= 2;

        var sql = $"INSERT INTO {tableName} ({columns}) VALUES ({values});";

        return sql;
    }
}
